/*
 * Enhydra Java Application Server
 * The Initial Developer of the Original Code is Lutris Technologies Inc.
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc.
 * All Rights Reserved.
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.
 *
 */


package calculator.presentation;

import java.math.BigDecimal;
import com.lutris.util.*;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import calculator.CalcApp;
import org.w3c.dom.*;
import org.w3c.dom.html.*;


/**
  *  A sample Enhydra application. A simple calculator is simulated using
  *  a single presentation object. Every time a button is pressed this
  *  presentation object is notified via the querry string
  *  "?button=...".
  *
  *  The responsability of this object is to emit an HTML string that
  *  shows the current state of the calculator. If a button is being
  *  passed to us in the query string, we process that button first,
  *  then show the current state of the calculator.
  *
  *  Because the SessionData object is being used to store the state of
  *  the calculator, multiple users may  use the calculator simultaneously
  *  "for free".
  *
  *  This method is called to process a button push. If no button
  *  was sent in the URL, do nothing.
  *
  *  @author Andy John andy@enhydra.org as a Jolt demo program
  *  Modified by Yaping Chen yaping.chen@lutris.com as a XMLC program
  */
public class Calculator implements HttpPresentation {

    /**
     *  Add a digit to the current display. We might be looking at a
     *  "stale" number (if overwrite is set). If so, then clear the
     *  display and save the current number for later use.
     */
    private static void addDigit(SessionData sd, String button)
                        throws KeywordValueException {
        String digits = (String) sd.get("digits");
        if (digits == null)
            digits = "0";
        if (sd.get("overwrite") != null) {
            try {
                sd.set("pending_num", new Float(digits));
            } catch (NumberFormatException e) {


            }
            digits = "";
            sd.remove("overwrite");
        }
        digits += button;
        sd.set("digits", digits);
    }


    /**
     *  Add a decimal point to the current display. If it already has a
     *  decimal point, then do nothing.
     */
    private static void addPoint(SessionData sd)
                        throws KeywordValueException {
        String digits = (String) sd.get("digits");
        if (digits == null)
            digits = "0";
        if (sd.get("overwrite") != null) {
            try {
                sd.set("pending_num", new Float(digits));
            } catch (NumberFormatException e) {
            }
            digits = "";
            sd.remove("overwrite");
        }
        if (digits.indexOf(".") != -1)
            return;
        digits += ".";
        sd.set("digits", digits);
    }


    /**
     *  Flip the sign of the number currently in the display.
     */
    private static void negate(SessionData sd)
                        throws KeywordValueException {
        String digits = (String) sd.get("digits");
        if (digits == null)
            return;
        if (digits.startsWith("-"))
            digits = digits.substring(1);
        else
            digits = "-" + digits;
        sd.set("digits", digits);
    }


    /**
     *  If there is a "pending number" (not in the current display),
     *  and there is a "pending operation" (add, subtract etc...),
     *  then use them and the current contents of the display and do some
     *  math. Leave the result in the current display.
     */
    private static void doEquals(SessionData sd)
                        throws KeywordValueException {
        String op = (String) sd.get("pending_op");
        if (op == null)
            return;
        Float f = (Float) sd.get("pending_num");
        if (f == null)
            f = new Float("0.0");
        float f2 = f.floatValue();
        //
        String digits = (String) sd.get("digits");
        if (digits == null)
            digits = "0";
        Float f3 = new Float(0);
        try {
            f3 = new Float(digits);
        } catch (NumberFormatException e) {
        }
        float f4 = f3.floatValue();
        double result = 0.0;
        //
        if (op.equals("+")) {
            result = f2 + f4;
        } else if (op.equals("-")) {
            result = f2 - f4;
        } else if (op.equals("*")) {
            result = f2 * f4;
        } else if (op.equals("/")) {
            result = f2 / f4;
        }
        //
        try
        {
          BigDecimal bd = new BigDecimal(result);
          sd.set("digits", bd.toString());
        } catch (Exception x) {
          // Just display a row of minuses, since there is no alpha display.
          sd.set("digits", "-------");
        }
        sd.remove("pending_op");
        sd.remove("pending_num");
        sd.set("overwrite", Boolean.TRUE);
    }


    /**
     *  Remember that a math function was pushed.
     */
    private static void doFunction(SessionData sd, String op)
                        throws KeywordValueException {
        if (sd.get("pending_num") != null) {
            doEquals(sd);
        } else {
            String digits = (String) sd.get("digits");
            if (digits == null)
                digits = "0";
        }
        sd.set("pending_op", op);
        sd.set("overwrite", Boolean.TRUE);
    }


    /**
     *  This method initializes an XMLC display field with the contents of
     *  the current number in the display. The string of digits is converted
     *  into a sequence of HTML image tags. Because the Enhydra table field
     *  contains HTML, we must use the "raw" option when using the
     *  Enhydra table field or some of the characters will be quoted for us.
     *
     *  This is only a demo program. It does not handle exceptional cases
     *  well, like dividing by zero, overflow, or infinties. The point is to
     *  show off one way to use Enhydra, not be a full-on HP calculator.
     */
    public static void getState(SessionData sd, CalculatorHTML page)
	                        throws Exception {

        // Convert current state to HTML
        String digits = (String) sd.get("digits");
        if (digits == null)
            digits = "0";
        // If it's an integer, don't show the decimal point.
        if (digits.endsWith(".0"))
            digits = digits.substring(0, digits.length()-2);
        // If it's too long to show, trim it or just show 0.
        // An error indication would be nice to add (hint hint... :)
        if (digits.indexOf(".") != -1) {
            if (digits.length() > 8)
                digits = digits.substring(0,8); // Truncate
        } else {
            if (digits.length() > 7)
                digits = "0"; // Overflow.
        }
        // Convert the string of digits to HTML.
	Element digitElement = page.getElementDigits();
        for (int i=0; i<digits.length(); i++) {
	    Element oneDigit = page.createElement("img");
            char c = digits.charAt(i);
            if (c == '0') {
		oneDigit.setAttribute("src", "media/zero.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '1') {
		oneDigit.setAttribute("src", "media/one.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '2') {
		oneDigit.setAttribute("src", "media/two.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '3') {
		oneDigit.setAttribute("src", "media/three.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '4') {
		oneDigit.setAttribute("src", "media/four.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '5') {
		oneDigit.setAttribute("src", "media/five.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '6') {
		oneDigit.setAttribute("src", "media/six.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '7') {
		oneDigit.setAttribute("src", "media/seven.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '8') {
		oneDigit.setAttribute("src", "media/eight.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '9') {
		oneDigit.setAttribute("src", "media/nine.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '-') {
		oneDigit.setAttribute("src", "media/dash.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
            else if (c == '.') {
		oneDigit.setAttribute("src", "media/point.jpeg");
		digitElement.insertBefore(oneDigit, null);
	    }
        }
        if (digits.length() == 0) {
	    Element oneDigit = page.createElement("img");
	    oneDigit.setAttribute("src", "media/zero.jpeg");
	    digitElement.insertBefore(oneDigit, null);
	}
    }

    /**
     * The run method that creates the dynamic page and handles dynamic items
     */
    public void run(HttpPresentationComms comms) throws Exception {

       // Get the session data for the user who issued this request.
       SessionData sd = comms.session.getSessionData();
       String button = (String) comms.request.getParameter("button");

       // do nothing if button is null
       if (button != null) {
           // Increment the counter stored in the application object.
           CalcApp myApplication = (calculator.CalcApp) comms.application;
           myApplication.buttonsPushed++;
           if (button.equals("0") || button.equals("1") ||
               button.equals("2") || button.equals("3") ||
               button.equals("4") || button.equals("5") ||
               button.equals("6") || button.equals("7") ||
               button.equals("8") || button.equals("9"))
	       addDigit(sd, button);
           else if (button.equals("point"))
               addPoint(sd);
           else if (button.equals("negate"))
               negate(sd);
           else if (button.equals("clear")) {
               sd.set("digits", "0");
               sd.remove("pending_op");
               sd.set("pending_num", new Float(0.0));
               sd.set("overwrite", Boolean.TRUE);
           } else if (button.equals("equals"))
               doEquals(sd);
           else if (button.equals("plus"))
               doFunction(sd, "+");
           else if (button.equals("minus"))
               doFunction(sd, "-");
           else if (button.equals("times"))
               doFunction(sd, "*");
           else if (button.equals("divide"))
               doFunction(sd, "/");
        }
        CalculatorHTML page = new CalculatorHTML();

	// setup digit images
        this.getState(sd, page);

	// write the page out
        comms.response.writeHTML(page.toDocument());
    }
}
