/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * 
 */

package demoApp.business.user;

import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import demoApp.data.user.*;
import demoApp.DemoApp;

/**
 * The User Manager Business Object.  This object handles requests
 * for user specific data and login verification.
 *
 * @author	Shawn McMurdo
 * @version	$Revision: 1.19 $
 */
public class DemoAppUserManager {
    /**
     * Username and password verification was successful.
     */
    public static final int AUTH_OK = 0;

    /**
     * Username and password verification failed because of
     * authentication failure.
     * AUTH_FAILED is thrown by a login request if the username
     * does not exist in the User database, the password was
     * incorrect, or an otherwise unknown error occured during the login
     * request.  The reason for the third case is that it is not wise from a
     * security standpoint to give unknown users potentially valuable
     * information about possible errors occuring in the login facility.
     */
    public static final int AUTH_FAILED = 1;

    /**
     * UNKNOWN_ERROR is indicated by methods other than the user login method
     * to report that an unknown error has prevented a request from
     * being processed.  The UserManager.login() method reports
     * AUTH_FAILED for unknown errors for security reasons.
     */
    public static final int UNKNOWN_ERROR = 2;

    /**
     * IO_ERROR is thrown by methods of the UserManager if a
     * communication failure prevented access to required information
     * from the User database or elsewhere.
     */
    public static final int IO_ERROR = 3;
    
    /**
     * PERMISSION_DENIED indicates that a UserManager method has
     * failed because the requesting user has insufficient privilege
     * to perform the requested operation.
     */
    public static final int PERMISSION_DENIED = 4;
    
    /**
     * MULTIPLE_LOGIN indicates that a user has attempted to login
     * more than the maximum number of allowable sessions.
     */
    public static final int MULTIPLE_LOGIN = 5;
    
    /**
     * ACCOUNT_DISABLED indicates that the account for the
     * requested user has been disabled by the administrator.
     */
    public static final int ACCOUNT_DISABLED = 6;
    
    /**
     * Process an authentication request.  Verify
     * that the user's account and password are valid.
     *
     * @param username The username for the account.
     * @param password The password entered by the user.
     *
     * @returns An int representing success (AUTH_OK), or a failure
     * reason (AUTH_FAILED, AUTH_IO_ERROR).
     */
    public int authenticate(String username, String password) {
        
	return ((DemoAppUser)getUser(username)).authenticate(password);
    }
    
    /**
     * Return a user business object that implements User for the 
     * given username.
     *
     * @param username The username for the account.
     * @return A user business object that implements User.
     */
    public User getUser(String username) {
	// Get the real user data object.
	// Return application's user object cast to User.
	return (User) getDemoAppUser(username);
    }

    /**
     * Process a login request issued by a user.  Verify
     * that the user's account and password are valid, that the user
     * is allowed to log in, and that the user has not logged in too
     * many times.  Issue a cookie corresponding to a registered
     * session.
     *
     * @param username The user identification entered by the user.
     * @param password The password entered by the user.
     * @param session The session object for the login session.  This
     *          should be a BasicSession object or one derived from it.
     * @exception LoginException Thrown if authentication fails.
     * @exception SessionException it there is a problem with the session.
     */
    public void login(String username, String password, Session session)
        throws LoginException, SessionException {
        StandardSession stdSession = (StandardSession) session;
	StandardSessionManager stdSessionManager =
            (StandardSessionManager)session.getSessionManager();

	// Verify the login.
	switch (authenticate(username, password)) {
	    case DemoAppUserManager.AUTH_OK:
		break; // Continue with login process.
	    case DemoAppUserManager.AUTH_FAILED:
		refuse(DemoAppUserManager.AUTH_FAILED);
	    case DemoAppUserManager.IO_ERROR:
		refuse(DemoAppUserManager.IO_ERROR);
	    default:
		refuse(DemoAppUserManager.UNKNOWN_ERROR);
	}
	
	// Bind the user to this session.
        // If the user was previously logged in, copy all
        // data into this session and log out the
        // other session.
        User user = getUser(username);
        if (!SessionUtil.resumeLogIn(user, stdSession)) {
            SessionUtil.logIn(user, session, false);
        }
    }
    
    /**
     * Terminates a user session.  The specified session
     * is removed from the active session table.
     * 
     * @param session Session to logout
     * @exception SessionException if the session could not be deleted.
     */
    public void logout(Session session) throws SessionException {
        SessionUtil.logOut(session);
    }
    
    /**
     * Return a DemoApp user business object for the given username.
     *
     * @param username The username for the account.
     * @return A DemoApp user business object.
     */
    public DemoAppUser getDemoAppUser(String username) {
        //FIX: Should look this up rather than create new instances
	// Get the real user data object.
	return new DemoAppUser(new DemoAppUserDO(username,
	    "demo", 1));
    }

    /**
     * Internal convenience method to throw a login refusal error.
     * 
     * @param why The error code indicating why the request failed.
     */
    private static final void refuse(int why)
        throws LoginException {
	throw new LoginException(why);
    }
}

